(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
	(factory((global.GoLib = global.GoLib || {}),global.React));
}(this, (function (exports,React) { 'use strict';

React = 'default' in React ? React['default'] : React;

var BLACK = 'B';

var EMPTY = 'EMPTY';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};











var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var LINE_WIDTH_FACTOR = 20;
var FONT_SIZE_FACTOR = 2.5;
var STAR_POINT_SIZE_FACTOR = 8;
var BACKGROUND_COLOR = 'rgb(251, 196, 103)';

var colorBlack = function colorBlack() {
  var opacity = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
  return 'rgba(0, 0, 0, ' + opacity;
};
var colorWhite = function colorWhite() {
  var opacity = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
  return 'rgba(255, 255, 255, ' + opacity;
};

// BoardDrawer exposes functions for drawing a go board on a 2d context

var BoardDrawer = function () {
  function BoardDrawer(boardSize, width, height) {
    classCallCheck(this, BoardDrawer);

    this.boxSize = width / (boardSize + 1);
    this.lineWidth = this.boxSize / LINE_WIDTH_FACTOR;
    this.fontSize = this.boxSize / FONT_SIZE_FACTOR;
    this.width = width;
    this.height = height;
    this.boardSize = boardSize;
    this.stars = this.calculateStars();
  }

  BoardDrawer.prototype.calculateStars = function calculateStars() {
    var boardSize = this.boardSize,
        boxSize = this.boxSize;

    var starPos = boardSize === 9 ? 3 : 4;
    var single = starPos * boxSize;
    var double = (boardSize + 1) / 2 * boxSize;
    var triple = (boardSize + 1 - starPos) * boxSize;
    var stars = [{ x: double, y: double }];

    if (boardSize % 2 !== 0 && boardSize > 7) {
      stars.push(
      // TopLeft
      { x: single, y: single },
      // Left
      { x: single, y: double },
      // BottomLeft
      { x: single, y: triple },
      // BottomMiddle
      { x: double, y: triple },
      // TopMiddle
      { x: double, y: single },
      // TopRight
      { x: triple, y: single },
      // MiddleRight
      { x: triple, y: double },
      // BottomRight
      { x: triple, y: triple });
    }

    return stars;
  };

  BoardDrawer.prototype.clear = function clear(ctx) {
    var width = this.width,
        height = this.height;


    ctx.save();
    ctx.clearRect(0, 0, width, height);
    ctx.fillStyle = BACKGROUND_COLOR;
    ctx.fillRect(0, 0, width, height);
    ctx.restore();
  };

  BoardDrawer.prototype.stones = function stones(ctx, moves) {
    var isHover = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var lineWidth = this.lineWidth,
        boxSize = this.boxSize;

    var opacity = isHover ? 0.7 : 1;

    ctx.save();
    ctx.lineWidth = lineWidth * 2;

    moves.forEach(function (move, index) {
      var moveX = move.coordinate.x + 1;
      var moveY = move.coordinate.y + 1;
      var isLastMove = index === moves.length - 1;
      var fillStyle = move.color === BLACK ? colorBlack(opacity) : colorWhite(opacity);

      // Draw stone
      ctx.fillStyle = fillStyle;
      ctx.beginPath();
      ctx.arc(moveX * boxSize, moveY * boxSize, boxSize / 2 - lineWidth, 0, 2 * Math.PI);
      if (!isHover) {
        ctx.stroke();
      }
      ctx.fill();

      // Mark last move
      if (isLastMove && !isHover) {
        var strokeStyle = move.color === BLACK ? colorWhite(opacity) : colorBlack(opacity);

        ctx.beginPath();
        ctx.strokeStyle = strokeStyle;
        ctx.lineWidth = lineWidth;
        ctx.arc(moveX * boxSize, moveY * boxSize, boxSize / 3.5, 0, 2 * Math.PI);
        ctx.stroke();
      }
    });

    ctx.restore();
  };

  BoardDrawer.prototype.hoverStones = function hoverStones(ctx, moves) {
    this.stones(ctx, moves, true);
  };

  BoardDrawer.prototype.grid = function grid(ctx) {
    var lineWidth = this.lineWidth,
        boxSize = this.boxSize,
        width = this.width,
        height = this.height,
        boardSize = this.boardSize;


    ctx.save();
    ctx.beginPath();
    ctx.lineWidth = lineWidth;

    for (var i = 1; i <= boardSize; i++) {
      ctx.moveTo(i * boxSize, boxSize);
      ctx.lineTo(i * boxSize, height - boxSize);
      ctx.moveTo(boxSize, i * boxSize);
      ctx.lineTo(width - boxSize, i * boxSize);
    }

    ctx.stroke();
    ctx.restore();
  };

  BoardDrawer.prototype.starPoints = function starPoints(ctx) {
    var boxSize = this.boxSize,
        boardSize = this.boardSize,
        stars = this.stars;


    ctx.save();
    ctx.beginPath();
    ctx.fillStyle = colorBlack();

    stars.forEach(function (pos) {
      ctx.moveTo(pos.x, pos.y);
      ctx.arc(pos.x, pos.y, boxSize / STAR_POINT_SIZE_FACTOR, 0, 2 * Math.PI);
    });

    ctx.fill();
    ctx.restore();
  };

  BoardDrawer.prototype.coordinates = function coordinates(ctx) {
    var boxSize = this.boxSize,
        fontSize = this.fontSize,
        boardSize = this.boardSize;


    ctx.save();
    ctx.font = 'bold ' + fontSize + 'px sans-serif';
    ctx.textAlign = 'center';
    ctx.fillStyle = colorBlack();

    for (var i = 1; i <= boardSize; i++) {
      ctx.fillText(String.fromCharCode(64 + i), i * boxSize, boxSize / 2);
      ctx.fillText('' + (boardSize - i + 1), boxSize / 2, i * boxSize + 3);
    }

    ctx.restore();
  };

  BoardDrawer.prototype.draw = function draw(ctx, moves, hoverMoves) {
    this.clear(ctx);
    this.grid(ctx);
    this.coordinates(ctx);
    this.starPoints(ctx);
    this.stones(ctx, moves);
    this.hoverStones(ctx, hoverMoves);
  };

  // calculateCoordinateFromMousePosition convert's a canvas mouse position to a board coordinate


  BoardDrawer.prototype.calculateCoordinateFromMousePosition = function calculateCoordinateFromMousePosition(mousePosition) {
    var x = Math.round(mousePosition.x / this.boxSize) - 1;
    var y = Math.round(mousePosition.y / this.boxSize) - 1;

    return { x: x, y: y };
  };

  return BoardDrawer;
}();

// calculateCanvasPosition takes clientX/Y and returns the relative clientX/Y of the canvas
var calculateCanvasPosition = function calculateCanvasPosition(canvas, x, y) {
  var bcr = canvas.getBoundingClientRect();

  return {
    x: x - bcr.left,
    y: y - bcr.top
  };
};

var isCoordinateOnBoard = function isCoordinateOnBoard(coordinate, boardSize) {
  return coordinate.x > 0 && coordinate.y > 0 && coordinate.x <= boardSize && coordinate.y <= boardSize;
};

// Board draw's a Go Board
var Board = function (_React$PureComponent) {
  inherits(Board, _React$PureComponent);

  function Board(props) {
    classCallCheck(this, Board);

    var _this = possibleConstructorReturn(this, _React$PureComponent.call(this, props));

    _this.handleMouseClick = function (e) {
      var mousePosition = calculateCanvasPosition(_this.canvas, e.clientX, e.clientY);

      _this.publishBoardPosition(mousePosition, _this.props.onPositionClick);
    };

    _this.handleTouchEnd = function (e) {
      e.preventDefault();

      var touch = e.changedTouches[0];
      var mousePosition = calculateCanvasPosition(_this.canvas, touch.clientX, touch.clientY);

      _this.publishBoardPosition(mousePosition, _this.props.onPositionClick);
    };

    _this.handleMouseMove = function (e) {
      var mousePosition = calculateCanvasPosition(_this.canvas, e.clientX, e.clientY);

      var boardCoordinate = _this.drawer.calculateCoordinateFromMousePosition(mousePosition);

      if (isCoordinateOnBoard(boardCoordinate, _this.drawer.boardSize)) {
        _this.props.onPositionHover(boardCoordinate);
      } else {
        _this.props.onPositionHover(null);
      }
    };

    _this.handleMouseLeave = function () {
      _this.props.onPositionHover(null);
    };

    var size = props.size,
        width = props.width,
        height = props.height;


    _this.drawer = new BoardDrawer(size, width, height);
    return _this;
  }

  Board.prototype.componentDidMount = function componentDidMount() {
    var _props = this.props,
        positions = _props.positions,
        width = _props.width,
        height = _props.height,
        hoverPositions = _props.hoverPositions;

    var context = this.canvas.getContext('2d');

    this.canvas.width = width;
    this.canvas.height = height;

    if (context) {
      this.context = context;
      this.drawer.draw(this.context, positions, hoverPositions);
    }
  };

  Board.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var positions = nextProps.positions,
        width = nextProps.width,
        height = nextProps.height,
        hoverPositions = nextProps.hoverPositions;


    if (this.canvas) {
      this.canvas.width = width;
      this.canvas.height = height;
    }

    if (this.context) {
      this.drawer.draw(this.context, positions, hoverPositions);
    }
  };

  Board.prototype.publishBoardPosition = function publishBoardPosition(cursorPosition, handler) {
    var boardCoordinate = this.drawer.calculateCoordinateFromMousePosition(cursorPosition);

    if (isCoordinateOnBoard(boardCoordinate, this.drawer.boardSize)) {
      handler(boardCoordinate);
    }
  };

  Board.prototype.render = function render() {
    var _this2 = this;

    return React.createElement('canvas', {
      ref: function ref(c) {
        return _this2.canvas = c;
      },
      onClick: this.handleMouseClick,
      onMouseMove: this.handleMouseMove,
      onTouchEnd: this.handleTouchEnd,
      onMouseLeave: this.handleMouseLeave
    });
  };

  return Board;
}(React.PureComponent);

Board.defaultProps = {
  size: 19,
  positions: [],
  hoverPositions: [],
  onPositionClick: function onPositionClick() {},
  onPositionHover: function onPositionHover() {}
};



var components = Object.freeze({
	Board: Board
});

// range creates an array of given size
var range = function range(size) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  return Array.from(Array(size), function (_, i) {
    return i + start;
  });
};

// eachBoardCoordinate call's `callback` for each board coordinate

var Board$2 = function () {
  function Board(boardSize) {
    classCallCheck(this, Board);
    this.data = [];

    this.boardSize = boardSize;
    this.data = range(boardSize * boardSize).map(function () {
      return EMPTY;
    });
  }

  Board.prototype.clear = function clear() {
    this.data = this.data.map(function () {
      return EMPTY;
    });
  };

  Board.prototype.setStatus = function setStatus(x, y, status) {
    var boardIndex = this.boardSize * x + y;

    this.data = this.data.map(function (value, i) {
      return i === boardIndex ? status : value;
    });
  };

  Board.prototype.getStatus = function getStatus(x, y) {
    var boardIndex = this.boardSize * x + y;

    return this.data[boardIndex];
  };

  return Board;
}();

var rand63 = function rand63() {
  return Math.floor(Math.random() * 9223372036854775807);
};

var Zobrist = function () {
  function Zobrist(tableSize) {
    classCallCheck(this, Zobrist);

    this.tableSize = tableSize;
    this.table = range(tableSize * tableSize).map(function (_) {
      return [rand63(), rand63()];
    });
    this.sum = 0;
  }

  Zobrist.prototype.hash = function hash(x, y, color) {
    var tableIndex = this.tableSize * x + y;
    var index = color === BLACK ? 0 : 1;

    this.sum ^= this.table[tableIndex][index];

    return this.sum;
  };

  Zobrist.prototype.getHash = function getHash() {
    return this.sum;
  };

  return Zobrist;
}();

exports.components = components;
exports.Board = Board$2;
exports.Zobrist = Zobrist;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=go-lib.js.map
