function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';

import BoardDrawer from './util/board-drawer';
import * as canvas from './util/canvas';
import * as rules from '../rules';

// Board draw's a Go Board
var Board = function (_React$PureComponent) {
  _inherits(Board, _React$PureComponent);

  function Board(props) {
    _classCallCheck(this, Board);

    var _this = _possibleConstructorReturn(this, _React$PureComponent.call(this, props));

    _this.handleMouseClick = function (e) {
      var mousePosition = canvas.calculateCanvasPosition(_this.canvas, e.clientX, e.clientY);

      _this.publishBoardPosition(mousePosition, _this.props.onPositionClick);
    };

    _this.handleTouchEnd = function (e) {
      e.preventDefault();

      var touch = e.changedTouches[0];
      var mousePosition = canvas.calculateCanvasPosition(_this.canvas, touch.clientX, touch.clientY);

      _this.publishBoardPosition(mousePosition, _this.props.onPositionClick);
    };

    _this.handleMouseMove = function (e) {
      var mousePosition = canvas.calculateCanvasPosition(_this.canvas, e.clientX, e.clientY);

      var boardCoordinate = _this.drawer.calculateCoordinateFromMousePosition(mousePosition);

      if (rules.isCoordinateOnBoard(boardCoordinate, _this.drawer.boardSize)) {
        _this.props.onPositionHover(boardCoordinate);
      } else {
        _this.props.onPositionHover(null);
      }
    };

    _this.handleMouseLeave = function () {
      _this.props.onPositionHover(null);
    };

    var size = props.size,
        width = props.width,
        height = props.height;


    _this.drawer = new BoardDrawer(size, width, height);
    return _this;
  }

  Board.prototype.componentDidMount = function componentDidMount() {
    var _props = this.props,
        positions = _props.positions,
        width = _props.width,
        height = _props.height,
        hoverPositions = _props.hoverPositions;

    var context = this.canvas.getContext('2d');

    this.canvas.width = width;
    this.canvas.height = height;

    if (context) {
      this.context = context;
      this.drawer.draw(this.context, positions, hoverPositions);
    }
  };

  Board.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var positions = nextProps.positions,
        width = nextProps.width,
        height = nextProps.height,
        hoverPositions = nextProps.hoverPositions;


    if (this.canvas) {
      this.canvas.width = width;
      this.canvas.height = height;
    }

    if (this.context) {
      this.drawer.draw(this.context, positions, hoverPositions);
    }
  };

  Board.prototype.publishBoardPosition = function publishBoardPosition(cursorPosition, handler) {
    var boardCoordinate = this.drawer.calculateCoordinateFromMousePosition(cursorPosition);

    if (rules.isCoordinateOnBoard(boardCoordinate, this.drawer.boardSize)) {
      handler(boardCoordinate);
    }
  };

  Board.prototype.render = function render() {
    var _this2 = this;

    return React.createElement('canvas', {
      ref: function ref(c) {
        return _this2.canvas = c;
      },
      onClick: this.handleMouseClick,
      onMouseMove: this.handleMouseMove,
      onTouchEnd: this.handleTouchEnd,
      onMouseLeave: this.handleMouseLeave
    });
  };

  return Board;
}(React.PureComponent);

Board.defaultProps = {
  size: 19,
  positions: [],
  hoverPositions: [],
  onPositionClick: function onPositionClick() {},
  onPositionHover: function onPositionHover() {}
};


export default Board;